/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "experimenter/operator_selector.h"
#include "libtkdb/exception.h"

using namespace Tk::Experimenter;

OperatorSelector:: OperatorSelector(Db::Database * database)
  : DomainSelector(database, "operator", true)
{
}

Tk::String
OperatorSelector:: render_entity(const Tk::Db::Id &entity_id)
{
  try
    {
      Tk::Db::Data::Value anonymous( _database->get_property("operator", entity_id, "anonymous") );
      
      if (boost::get<bool>(anonymous.get_datum().get_value()))
	return Tk::String("Anonymous operator #") + Tk::String(entity_id);
    }
  catch (Tk::Db::PropertyNotSet pns) {}
  
  Tk::String name, email;
  try
    {
      Tk::Db::Data::Value val( _database->get_property("operator", entity_id, "name") );
      name = boost::get<Tk::String>(val.get_datum().get_value());
    }
  catch (Tk::Db::PropertyNotSet pns) {}
  try
    {
      Tk::Db::Data::Value val( _database->get_property("operator", entity_id, "email") );
      email = boost::get<Tk::String>(val.get_datum().get_value());
    }
  catch (Tk::Db::PropertyNotSet pns) {}
  
  if (name.length() > 1 && email.length() > 1)
    return name + Tk::String(" <") + email + Tk::String(">");
  
  if (name.length() > 1)
    return name;
  
  if (email.length() > 1)
    return email;
  
  return Tk::String("Operator #") + Tk::String(entity_id);
}

Tk::Db::Ids
OperatorSelector:: get_operators()
{
  return get_selected();
}
