/* 
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2007 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "experimenter/speed.h"

using namespace Tk;
using namespace Experimenter;

Speed:: Speed()
  : _adjustment(0.1, 0.0, 10.0, 0.001, 0.01),
    _scroll("Scroll")
{
  set_spacing(8);
  
  pack_start(_scroll, false, true);
  _scroll.set_active();
  _scroll.signal_toggled().connect( sigc::mem_fun(*this, &Speed::scroll_toggled) );
  _scroll.set_tooltip_text("Whether to scroll the screen in order to draw many rows or just draw down a single screen without scrolling");
  
  pack_start(_speed, true, true);
  _speed.set_adjustment(_adjustment);
  _speed.set_digits(4);
  _speed.set_tooltip_text("How quickly to scroll the screen, in screens per second.  NB: only low values are practical due to hardware speed");
}

void
Speed:: scroll_toggled()
{
  bool scrolled( get_scrolled() );
  _speed.set_sensitive(scrolled);
  _signal_scroll_changed.emit(scrolled);
}

Speed::ScrollChanged *
Speed:: signal_scroll_changed()
{
  return &_signal_scroll_changed;
}

double
Speed:: get_speed() const
{
  return _speed.get_value();
}

bool
Speed::get_scrolled() const
{
  return _scroll.get_active();
}
