/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "libtkdb/bytes.h"

using namespace Tk::Db;

Bytes:: Bytes()
{
}

Bytes:: Bytes(const Bytes &other)
  : odbc::Bytes(other)
{
}

Bytes:: Bytes(const ODBCXX_SIGNED_CHAR_TYPE *data, size_t dataLen)
  : odbc::Bytes(data, dataLen)
{
}

Bytes:: Bytes(std::istream &stream, std::string::size_type *len)
{
  Tk::Stream *base_strm;
  if (len)
    base_strm = new Tk::Stream(stream, *len);
  else
    base_strm = new Tk::Stream(stream);
  
  Tk::BasicStream<ODBCXX_SIGNED_CHAR_TYPE> *conv_strm( base_strm->write_stream<ODBCXX_SIGNED_CHAR_TYPE>() );
  delete base_strm;
  
  size_t conv_len( conv_strm->get_len() );
  ODBCXX_SIGNED_CHAR_TYPE conv_buf[conv_len];
  conv_strm->get_stream()->read(conv_buf, conv_len);
  delete conv_strm;
  
  *this = Bytes(conv_buf, conv_len);
}


Bytes:: ~Bytes()
{
  for (StreamList::iterator iter( _gotten_streams.begin() );
       iter != _gotten_streams.end();
       ++iter)
    delete *iter;
}

Tk::Stream *
Bytes:: get_stream()
{
  Tk::BasicStream<ODBCXX_SIGNED_CHAR_TYPE> base_strm(getData(), getSize());
  Tk::Stream *stream( base_strm.write_stream<char>() );
  _gotten_streams.push_back(stream);
  return stream;
}
