/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <odbc++/drivermanager.h>
#include <odbc++/databasemetadata.h>

#include "libtkcommon/tkintl.h"
#include "libtkdb/connection.h"

using namespace Tk::Db;

Connection:: Connection (const Tk::String &data_source_name,
			 const Tk::String &username,
			 const Tk::String &password)
  : _connection(odbc::DriverManager::getConnection(data_source_name, username, password))
{
  //WDEBUG_OUT << L"Getting database connection metadata..." << std::endl;
  odbc::DatabaseMetaData *metadata( _connection->getMetaData() );
  //WDEBUG_OUT << L"...got database connection metadata" << std::endl;
  
  std::wcout << _(L"Database product: ")
	     << Tk::String(metadata->getDatabaseProductName())
	     << _(L"; version: ")
	     << Tk::String(metadata->getDatabaseProductVersion())
	     << std::endl
	     << _(L"Driver: ")
	     << Tk::String(metadata->getDriverName())
	     << _(L"; version: ")
	     << Tk::String(metadata->getDriverVersion())
	     << std::endl;
  
  //WDEBUG_OUT << L"Initialised Database" << std::endl;
}

Connection:: ~Connection()
{
  delete _connection;
}

Connection::Mutex &
Connection:: get_mutex()
{
  return _mutex;
}

Statement *
Connection:: new_statement()
{
  return _connection->createStatement(odbc::ResultSet::TYPE_FORWARD_ONLY,
				      odbc::ResultSet::CONCUR_READ_ONLY);
}

void
Connection:: execute_statement(const Tk::String &sql)
{
  StatementPtr stmt( new_statement() );
  stmt->execute(sql);
}

DataSet *
Connection:: execute_query(const Tk::String &sql)
{
  return new DataSet( new_statement()->executeQuery(sql) );
}

PStatement *
Connection:: prepare_statement(const Tk::String &sql)
{
  return _connection->prepareStatement(sql,
				       odbc::ResultSet::TYPE_FORWARD_ONLY,
				       odbc::ResultSet::CONCUR_READ_ONLY);
}

Id
Connection:: get_last_insert_id()
{
  Tk::String sql;
  
  Tk::String db_product( _connection->getMetaData()->getDatabaseProductName() );
  //WDEBUG_OUT << L"Database product name: " << db_product << std::endl;
  if (db_product == "SQLite")
    sql = "select last_insert_rowid()";
  else
    sql = "select last_insert_id()";

  Statement *stmt( new_statement() );
  DataSetPtr result( new DataSet(stmt->executeQuery(sql)) );
  
  bool valid( (*result)->next() );
  if (!valid)
    throw std::runtime_error("Could not get last insert id");
  
  return (*result)->getLong(1);
}

PStatement *
Connection:: get_table_insert_statement(const Tk::String &table, const Tk::Strings &columns)
{
  std::wostringstream query;
  std::wostringstream values;

  query << L"insert into " << table << L"(";
  values << L") values (";

  Tk::Strings::const_iterator iter, next;
  for(iter = columns.begin();
      iter != columns.end();
      iter = next)
    {
      next = iter; ++next;
      
      query << *iter;
      values << L"?";
      
      if(next != columns.end())
	{
	  query << L", ";
	  values << L", ";
	}
    }
  query << values.str();
  query << L")";
  
  return prepare_statement(query.str());
}

