/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_DISPLAY_H__
#define __TK_DISPLAY_H__

#include <list>
#include <vector>

#include <SDL/SDL.h>

#include "libtkcommon/pair.h"
#include "libtkcommon/tkvalue.h"
#include "types.h"

namespace Tk {

  class Display
  {
  private:
    typedef std::list<SDL_Rect> BlockList;
    typedef std::vector<Line>   LineList;
    
    SDL_Surface  * _sdl_display;
    Uint32         _white;
    Uint32         _black;
    unsigned       _trace_count;
    unsigned       _total_lines;
    unsigned       _print_count;
    int            _last_y;
    bool           _scrolling;
    unsigned       _horizon;
    LineList     * _x_history;

    int truncate_x(int x);
    int get_last_x(OffsetIndex index) const;
    void get_x_limits(OffsetIndex index, int &min, int &max);
    static unsigned calc_total_lines(unsigned total_time, double speed, unsigned height);
    
    void lock();
    void unlock();
    BlockList create_count_down_blocks(unsigned count);
    
  public:
    
    Display (unsigned height,
	     unsigned width,
	     unsigned trace_count,
	     unsigned total_time,
	     bool     scrolled = false,
	     double   speed = 0.0,
	     double   horizon = 0.0);
    ~Display();
  
    void count_down (unsigned count);
    void print_line (const Line &line);

    Line calc_start_line() const;
    unsigned get_height() const;
    unsigned get_width() const;
    unsigned get_total_lines() const;
  };

}; /* namespace Tk */

#endif /* __TK_DISPLAY_H__ */
